// ---------------------------------------------------------------------------
// auto_repair_kit.cs -- Version 3.15 -- October 27, 2000
// by Lorne Laliberte (writer@planetstarsiege.com)
//
// - automatically use a repair kit when you pick one up
// - heal yourself faster at inventory stations
// - (manually) taunt opponents who leave a repair kit behind
//
// Special thanks to Andrew Mather (a.k.a. ]|sh|[ Ripley) for suggesting
// the OneKeyMode
//
// http://www.writerscripts.com
// ---------------------------------------------------------------------------

include("writer\\version.cs");
version("writer\\auto_repair_kit.cs", "3.15", "Lorne Laliberte", "Oct 27, 2000");

include("writer\\event.cs");
include("writer\\station_events.cs");
include("writer\\inventory.cs");
include("writer\\item_events.cs");
include("writer\\loadout.cs");
include("writer\\save.cs");
include("writer\\schedule.cs");
include("writer\\screensize.cs");
include("writer\\whatif.cs");
include("writer\\you_respawned.cs");
include("writer\\you_spawned.cs");

// ---------------------------------------------------------------------------
// DEFAULT SETTINGS:
//
// Edit these if you're running this script for the first time
// All subsequent settings must be done in the config\wp_auto_repair_kit.cs file

    // Enable "one key mode" -- automatically switches to "Use Next Repair Kit"
    // mode when you press the use repair kit key and you aren't carrying a
    // repair kit.  The next repair kit you pick up will be used, and then
    // "Use Next Repair Kit" mode will be disabled.
    $AutoRepairKit::OneKeyMode = "true";

    // Display a message when you pick up a repair kit
    $AutoRepairKit::AlertWhenFound = "true";

    // Automatically buy repair kits to replenish your health when you access a
    // station in "Use Next Repair Kit" mode
    $AutoRepairKit::AutoBuyHealth = "true";

    // ALWAYS automatically buy repair kits to replenish your health when you
    // access a station
    $AutoRepairKit::AlwaysBuyHealth = "false";

    // Modifier key -- hold it to override the current AlwaysBuyHealth setting,
    // forcing AlwaysBuyHealth on if it's off, and forcing it off if it's on
    $AutoRepairKit::AlwaysBuyOverrideKey = "";

    // Message used when you press the repair kit taunt key
    $AutoRepairKit::Taunt::string = "Thanks for the repair kit!~wthanks";

    // Use cool bitmap text alerts instead of remoteBP's
    $AutoRepairKit::useBitmaps = "true";

    // Coordinates for the bitmap text alerts, if used
    $AutoRepairKit::x = "50%";
    $AutoRepairKit::y = "65%";

    // Coordinates for the bitmap text alerts when a rangefinder is active
    $AutoRepairKit::RangeX = "50%";
    $AutoRepairKit::RangeY = "30%";

    // Delay in seconds between buy/use attempts when buying repair kits
    // at an inventory station to replenish your health faster
    $AutoRepairKit::Delay = "0.1";

    // Use a repair kit
    //bind(keyboard0, make, "capslock", to, "useRepairKit();");
    //bind(keyboard0, break, "capslock", to, "", "useRepairKit_break");

    // Toggle Auto Use Repair Kit mode (overrides Use Next Repair Kit mode)
    //bind(keyboard0, make, l_shift, "capslock", to, "AutoRepairKit::toggle();");
    //bind(keyboard0, break, l_shift, "capslock", to, "", "AutoRepairKit::toggle_break");

    // Repair Kit Taunt ("Thanks for the repair kit!")
    //bind(keyboard0, make, alt, "capslock", to, "RepairKitTaunt();");
    //bind(keyboard0, break, alt, "capslock", to, "", "RepairKitTaunt_break");

// ---------------------------------------------------------------------------

WriterPrefs::load("auto_repair_kit");

// Toggle Auto Use Repair Kit mode on/off -- the only way I've found
// to do this client side is from onClientMessage().
// Setting $AutoUse[RepairKit] to true only works if you're serving.
//
function AutoRepairKit::Toggle()
{
    $AutoRepairKit::Override = $AutoRepairKit::useNext = !$AutoRepairKit::useNext;

    if($AutoRepairKit::Override)
    {
        if(getItemCount("Repair Kit"))
            AutoRepairKit::alert(autoOn);
        else
            AutoRepairKit::alert(nokitautoOn);
    }
    else
    {
        if(getItemCount("Repair Kit"))
            AutoRepairKit::alert(autoOff);
        else
            AutoRepairKit::alert(nokitautoOff);
    }
    Event::Trigger(eventAutoRepairKitToggled);
}


// Taunt enemy at press of a key
//
function RepairKitTaunt()
{
    say(0, $AutoRepairKit::Taunt::string);
}


$AutoRepairKit::width = 85;
$AutoRepairKit::height = 50;

$AutoRepairKit::bitmap[used] = "<b1,1:writer\\kitalert\\repairkitused.bmp>";
$AutoRepairKit::bitmap[found] = "<b1,1:writer\\kitalert\\repairkitfound.bmp>";
$AutoRepairKit::bitmap[nokitautoOn] = "<b1,1:writer\\kitalert\\nokitautoon.bmp>";
$AutoRepairKit::bitmap[nokitautoOff] = "<b1,1:writer\\kitalert\\nokitautooff.bmp>";
$AutoRepairKit::bitmap[autoOn] = "<b1,1:writer\\kitalert\\autouseon.bmp>";
$AutoRepairKit::bitmap[autoOff] = "<b1,1:writer\\kitalert\\autouseoff.bmp>";
$AutoRepairKit::bitmap[next] = "<b1,1:writer\\kitalert\\autousenext.bmp>";
$AutoRepairKit::bitmap[none] = "<b1,1:writer\\kitalert\\norepairkit.bmp>";


function AutoRepairKit::alert(%alert, %time)
{
    if(%time == "")
        %time = 2;

    if($AutoRepairKit::useBitmaps)
    {
        $AutoRepairKit::currentText = $AutoRepairKit::bitmap[%alert];
        wHUD::show(kitAlert, $AutoRepairKit::bitmap[%alert]);
        schedule::add("wHUD::hide(kitAlert);", %time);
        return;
    }

    if(%alert == used)
        remoteBP(2048, "<L5><f1>Repair Kit used!", %time);
    else if(%alert == found)
        remoteBP(2048, "<L5>You <f1>found<f0> a <f1>repair kit", %time);
    else if(%alert == next)
        remoteBP(2048, "<L5><f1>No Repair Kit<f0> -- <f1>will auto<f0>matically <f1>use next<f0> kit found", 4);
    else if(%alert == nokitautoOn)
        remoteBP(2048, "<L5><f1>No Repair Kit<f0> -- but <f1>auto-use<f0> repair kit is <f1>ON", 4);
    else if(%alert == nokitautoOff)
        remoteBP(2048, "<L5><f1>No Repair Kit<f0> -- and <f1>auto-use<f0> repair kit is <f1>OFF", 4);
    else if(%alert == autoOn)
        remoteBP(2048, "<L5>Auto Use Repair Kit <f2>ON", 4);
    else if(%alert == autoOff)
        remoteBP(2048, "<L5>Auto Use Repair Kit <f2>OFF", 4);
    else if(%alert == none)
        remoteBP(2048, "<L5><f1>No Repair Kit", 4);
}


// Automatically use repair kit when one is picked up
//
function AutoRepairKit::onYouReceived_Kit(%number, %msg)
{
    if($AutoRepairKit::useNext)
    {
        if(!$AutoRepairKit::Override)
            $AutoRepairKit::useNext = false;

        newUseItem($Inv::Repair_Kit);
        AutoRepairKit::alert(used); // added in case getItemCount doesn't see the repair kit yet in the onUse function below
    }
    else
    {
        if($AutoRepairKit::AlertWhenFound)
            AutoRepairKit::alert(found);

        Event::Trigger(eventRepairKitFound);
    }

	return;
}

function AutoRepairKit::onCouldNotBuy_Kit()
{
    return mute;
}

// Show a message when a repair kit is used -- and if you're at an inventory
// station, use and buy as many repair kits as you need to get back to full
// health.  So, no more hogging the inv stations like a wounded soldier!
// Just hit your normal use repair kit key, wait for the little "buyItem" bell
// sounds to stop ringing, and walk away.
//
function AutoRepairKit::onUse(%itemtype, %itemname)
{
    if(%itemtype == $Inv::Repair_Kit)
    {
        if($Station::AtStation)
        {
            remoteEval(2048, buyItem, $Inv::Repair_Kit);

            loadout::armor();

            $useRepairKit::count++;

            if( ($loadout::carrying["Light Armor"]) && $useRepairKit::count > 3)
            {
                Event::Trigger(eventFullHealthBought);
                $useRepairKit::count = 0;
                return mute;
            }

            if( ($loadout::carrying["Medium Armor"]) && $useRepairKit::count > 4)
            {
                Event::Trigger(eventFullHealthBought);
                $useRepairKit::count = 0;
                return mute;
            }

            if($useRepairKit::count > 6)
            {
                Event::Trigger(eventFullHealthBought);
                $useRepairKit::count = 0;
                return mute;
            }

            if($useRepairKit::count == 1)
                Event::Trigger(eventFullHealthBuying);

            schedule::add("use(\"Repair Kit\");", $AutoRepairKit::Delay);
        }
        else
        {
            if(getItemCount("Repair Kit"))
                AutoRepairKit::alert(used);
        }
    }
    return;
}


function useRepairKit()
{
    if($Station::AtStation || getItemCount("Repair Kit"))
    {
        newUseItem($Inv::Repair_Kit);
    }
    else
    {
        if($AutoRepairKit::Override)
        {
            if(getItemCount("Repair Kit"))
                AutoRepairKit::alert(autoOn);
            else
                AutoRepairKit::alert(nokitautoOn);
        }
        else if($AutoRepairKit::OneKeyMode)
        {
            AutoRepairKit::alert(next);
            $AutoRepairKit::useNext = true;
        }

        Event::Trigger(eventRepairKitPressed);
    }
}


function AutoRepairKit::onEnterStation()
{
    if($AutoRepairKit::AlwaysBuyHealth)
        useRepairKit();
    else if($AutoRepairKit::useNext)
    {
        if(!getItemCount("Repair Kit"))
            useRepairKit();
    }
    return;
}


function AutoRepairKit::onYouSpawned()
{
    // Build kitAlert on first weapon switch, after playGui exists
    if(!wHUD::exists(kitAlert))
        wHUD::make(kitAlert, $AutoRepairKit::x, $AutoRepairKit::y, $AutoRepairKit::width, $AutoRepairKit::height, 1, 0);

    return;
}

function AutoRepairKit::relocate()
{
    wHUD::make(kitAlert, $AutoRepairKit::x, $AutoRepairKit::y, $AutoRepairKit::width, $AutoRepairKit::height, 1, 0);
    return;
}


function AutoRepairKit::onRangeFinderOn()
{
    if(($AutoRepairKit::x == $AutoRepairKit::rangeX) && ($AutoRepairKit::y == $AutoRepairKit::rangeY))
        return;

    wHUD::make(kitAlert, $AutoRepairKit::rangeX, $AutoRepairKit::rangeY, $AutoRepairKit::width, $AutoRepairKit::height, 1, wHUD::isVisible(kitAlert), $AutoRepairKit::currentText);
}


function AutoRepairKit::onRangeFinderOff()
{
    if(($AutoRepairKit::x == $AutoRepairKit::rangeX) && ($AutoRepairKit::y == $AutoRepairKit::rangeY))
        return;

    wHUD::make(kitAlert, $AutoRepairKit::x, $AutoRepairKit::y, $AutoRepairKit::width, $AutoRepairKit::height, 1, wHUD::isVisible(kitAlert), $AutoRepairKit::currentText);
}

function auto_repair_kit::save()
{
    AddExportText("    // Enable \"one key mode\" -- automatically switches to \"Use Next Repair Kit\"");
    AddExportText("    // mode when you press the use repair kit key and you aren't carrying a");
    AddExportText("    // repair kit.  The next repair kit you pick up will be used, and then");
    AddExportText("    // \"Use Next Repair Kit\" mode will be disabled.");
    ExportSetting("    $AutoRepairKit::OneKeyMode", true);
    AddExportText("");
    AddExportText("    // Display a message when you pick up a repair kit");
    ExportSetting("    $AutoRepairKit::AlertWhenFound", true);
    AddExportText("");
    AddExportText("    // Automatically buy repair kits to replenish your health when you access a");
    AddExportText("    // station in \"Use Next Repair Kit\" mode");
    ExportSetting("    $AutoRepairKit::AutoBuyHealth", true);
    AddExportText("");
    AddExportText("    // ALWAYS automatically buy repair kits to replenish your health when you");
    AddExportText("    // access a station");
    ExportSetting("    $AutoRepairKit::AlwaysBuyHealth", false);
    AddExportText("");
    AddExportText("    // Modifier key -- hold it to override the current AlwaysBuyHealth setting,");
    AddExportText("    // forcing AlwaysBuyHealth on if it's off, and forcing it off if it's on");
    ExportSetting("    $AutoRepairKit::AlwaysBuyOverrideKey", "");
    AddExportText("");
    AddExportText("    // Message used when you press the repair kit taunt key");
    ExportSetting("    $AutoRepairKit::Taunt::string", "Thanks for the repair kit!~wthanks");
    AddExportText("");
    AddExportText("    // Use cool bitmap text alerts instead of remoteBP's");
    ExportSetting("    $AutoRepairKit::useBitmaps", true);
    AddExportText("");
    AddExportText("    // Coordinates for the bitmap text alerts, if used");
    ExportSetting("    $AutoRepairKit::x", "50%");
    ExportSetting("    $AutoRepairKit::y", "65%");
    AddExportText("");
    AddExportText("    // Coordinates for the bitmap text alerts when a rangefinder is active");
    ExportSetting("    $AutoRepairKit::RangeX", "50%");
    ExportSetting("    $AutoRepairKit::RangeY", "30%");
    AddExportText("");
    AddExportText("    // Delay in seconds between buy/use attempts when buying repair kits");
    AddExportText("    // at an inventory station to replenish your health faster");
    ExportSetting("    $AutoRepairKit::Delay", "0.1");
    AddExportText("");
    AddExportText("    // Use a repair kit");

    ExportBind("useRepairKit();", "//bind(keyboard0, make, \"capslock\", TO, \"useRepairKit();\");");
    ExportBind("useRepairKit_break", "//bind(keyboard0, break, \"capslock\", TO, \"\", useRepairKit_break);");

    AddExportText("");
    AddExportText("    // Toggle Auto Use Repair Kit mode (overrides Use Next Repair Kit mode)");

    ExportBind("AutoRepairKit::toggle();", "//bind(keyboard0, make, l_shift, \"capslock\", TO, \"AutoRepairKit::toggle();\");");
    ExportBind("AutoRepairKit::toggle_break", "//bind(keyboard0, break, l_shift, \"capslock\", TO, \"\", AutoRepairKit::toggle_break);");

    AddExportText("");
    AddExportText("    // Repair Kit Taunt (\"Thanks for the repair kit!\")");

    ExportBind("RepairKitTaunt();", "//bind(keyboard0, make, alt, \"capslock\", TO, \"RepairKitTaunt();\");");
    ExportBind("RepairKitTaunt_break", "//bind(keyboard0, break, alt, \"capslock\", TO, \"\", RepairKitTaunt_break);");
}

WriterPrefs::save("auto_repair_kit");

function auto_repair_kit::init()
{
    needs("writer\\auto_repair_kit.cs", "writer\\event.cs 2.7",
                                        "writer\\inventory.cs 3.3",
                                        "writer\\station_events.cs 4.8",
                                        "writer\\item_events.cs 2.12",
                                        "writer\\loadout.cs 1.0",
                                        "writer\\save.cs 2.3",
                                        "writer\\schedule.cs 1.3",
                                        "writer\\screensize.cs 1.1",
                                        "writer\\whatif.cs 1.1",
                                        "writer\\you_respawned.cs 1.2",
                                        "writer\\you_spawned.cs 1.1");

    // Toggle Auto Repair Kit mode off after resupplying at a station
    Event::Attach(eventResupplyComplete, "if(!$AutoRepairKit::Override) $AutoRepairKit::useNext=false;");
    Event::Attach(eventYouRespawned, "if(!$AutoRepairKit::Override) $AutoRepairKit::useNext=false;");

    Event::Attach(eventUse, AutoRepairKit::onUse);
    Event::Attach(eventExitStation, "$useRepairKit::count=0;");

    Event::Attach(eventRangeFinderOn, "AutoRepairKit::onRangeFinderOn();");
    Event::Attach(eventRangeFinderOff, "AutoRepairKit::onRangeFinderOff();");

    Event::Attach(eventYouSpawned, "AutoRepairKit::onYouSpawned();");
    Event::Attach(eventScreenModeChanged, "AutoRepairKit::relocate();");
    Event::Attach(eventScreenSizeChanged, "AutoRepairKit::relocate();");

    Event::Attach("eventCouldNotBuy_Repair Kit", "AutoRepairKit::onCouldNotBuy_Kit();");
    Event::Attach("eventYouReceived_Repair Kit", AutoRepairKit::onYouReceived_Kit);

    if($AutoRepairKit::AutoBuyHealth || $AutoRepairKit::AlwaysBuyHealth)
        Event::Attach(eventEnterStation, "AutoRepairKit::onEnterStation();");

    if(String::Compare($AutoRepairKit::AlwaysBuyOverrideKey, ""))
    {
        Modifier::bind($AutoRepairKit::AlwaysBuyOverrideKey);
        Event::Attach("modifier" @ $AutoRepairKit::AlwaysBuyOverrideKey @ Pressed, "$AutoRepairKit::AlwaysBuyHealth = !$AutoRepairKit::AlwaysBuyHealth;");
        Event::Attach("modifier" @ $AutoRepairKit::AlwaysBuyOverrideKey @ Released, "$AutoRepairKit::AlwaysBuyHealth = !$AutoRepairKit::AlwaysBuyHealth;");
    }
}
Event::Attach(writerInit, "auto_repair_kit::init();");

if(!$UsingWriterLoader) auto_repair_kit::init();
